/* A simple server in the internet domain using TCP
   The port number is passed as an argument */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sstream>
#include <unistd.h>
#include <iostream>
#include <sys/types.h> 
#include <sys/socket.h>
#include <netinet/in.h>
#include <vector>
#include "Board.h"
#include "Engine.h"
using namespace std;

void error(const char *msg)
{
    perror(msg);
    exit(1);
}

int main(int argc, char *argv[])
{
    int sockfd, newsockfd, newsockfd2, portno;
    socklen_t clilen, clilenen;
    
	struct sockaddr_in serv_addr, cli_addr, cli2_addr;

	int n;
    
	if (argc < 2) {
         fprintf(stderr,"ERROR, no port provided\n");
         exit(1);
    }
     
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
     
	if (sockfd < 0) 
       error("ERROR opening socket");
     
	bzero((char *) &serv_addr, sizeof(serv_addr));
     
	portno = atoi(argv[1]);
     
	serv_addr.sin_family = AF_INET;
     
	serv_addr.sin_addr.s_addr = INADDR_ANY;
     
	serv_addr.sin_port = htons(portno);
     
	if (bind(sockfd, (struct sockaddr *) &serv_addr,
            sizeof(serv_addr)) < 0) 
            error("ERROR on binding");
     
	listen(sockfd,5);
     
	clilen = sizeof(cli_addr);
     
	newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, &clilen);
     
	if (newsockfd < 0) 
        error("ERROR on accept");
	
	//After all the server setup is done, we start the board
	Engine e;
	string move;
	
	bool gameOver = false; 
	vector<Board> record;
	e.getBoard()->snapshot(record,(*e.getBoard()));
    char buffer[256];
    char info[256];
	int choice;
	int move_counter = 0;
	string final_move;
		
	//Ask client about game type
	string introduction = "WELCOME\n1. Play against AI?\n2. Watch AI vs. AI?3. Play Client vs. Client?\nEnter choice: \n";
	write(newsockfd, introduction.c_str(), introduction.length());
	n = read(newsockfd,info,255); //Reads choice as a string
	istringstream convert(info); //Converts the read string to an integer
	convert >> choice; //Sets value equal to the converted value
	//Later in the project, we need to check for AI and Human modes
	cout << "OK" << endl;
	bzero(info,256); //Resets info back to normal
	
	string choice_difficulty = "EASY";
	if(choice != 3) {
		string difficulty_select = "Choose Difficulty\n1. Easy\n2. Medium\n3. Hard\nEnter choice: \n";
		write(newsockfd, difficulty_select.c_str(), difficulty_select.length());
		n = read(newsockfd,info,255);
		istringstream con(info);
		con >> choice_difficulty;
		cout << "OK" << endl;
		bzero(info,256);
	}
	
	while(true) {
		
		if(choice == 3) {
			
			listen(sockfd,5);
			
			clilenen = sizeof(cli_addr);
			newsockfd2 = accept(sockfd, (struct sockaddr *) &cli2_addr, &clilenen);
			write(newsockfd2, "skip", 4);
			sleep(1);
			string tempboard = e.getBoard()->boardToString();
			write(newsockfd2, tempboard.c_str(), tempboard.length());//Display the board to the client (line by line)
			write(newsockfd, tempboard.c_str(), tempboard.length());//Display the board to the client (line by line)
			bool firstturn = true;
			
			if (newsockfd2 < 0) 
				error("ERROR on accept");
			
			while(gameOver != true)
			{
				gameOver = e.getBoard()->isGameOver();
				
				while(e.getBoard()->getTurn() == 'O' && !e.getBoard()->isValid())
				{
					e.getBoard()->displayBoard();//Display the board on the server
					string boardState = e.getBoard()->boardToString();
					final_move = e.getBoard()->boardToString();
					cout<<"\nWaiting for client: ";
					n = read(newsockfd,buffer,255);
					move = buffer;
					bzero(buffer,256);
					parse(move,(*e.getBoard()));
					
					if(e.getBoard()->isValid()) {
						boardState = e.getBoard()->boardToString();
						cout << boardState << "testing\n\n";
						write(newsockfd, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
						//write(newsockfd2, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
						e.getBoard()->changeTurns();
						e.getBoard()->setValidFalse();
					}
				}
				
				gameOver = e.getBoard()->isGameOver();
				e.getBoard()->setValidFalse();
				e.getBoard()->snapshot(record, *e.getBoard());
				
				while(e.getBoard()->getTurn() == 'X' && !e.getBoard()->isValid())
				{
					e.getBoard()->displayBoard();//Display the board on the server
					string boardState = e.getBoard()->boardToString();
					final_move = e.getBoard()->boardToString();
					write(newsockfd2, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
					cout<<"\nWaiting for client: ";
					n = read(newsockfd2,buffer,255);
					move = buffer;
					bzero(buffer,256);
					parse(move,(*e.getBoard()));
					
					if(e.getBoard()->isValid()) {
						boardState = e.getBoard()->boardToString();
						write(newsockfd, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
						write(newsockfd2, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
						e.getBoard()->changeTurns();
						e.getBoard()->setValidFalse();
					}
				}
				
				gameOver = e.getBoard()->isGameOver();
				e.getBoard()->setValidFalse();
				e.getBoard()->snapshot(record, *e.getBoard());
			}
		}
		
		else if(choice == 1) 
		{
			while(gameOver != true)
			{
				gameOver = e.getBoard()->isGameOver();
				
				while(e.getBoard()->getTurn() == 'O' && !e.getBoard()->isValid())
				{
					e.getBoard()->displayBoard();//Display the board on the server
					string boardState = e.getBoard()->boardToString();
					final_move = e.getBoard()->boardToString();
					write(newsockfd, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
					cout<<"\nWaiting for client: ";
					n = read(newsockfd,buffer,255);
					move = buffer;
					bzero(buffer,256);
					parse(move,(*e.getBoard()));
					
					if(e.getBoard()->isValid()) {
						e.getBoard()->changeTurns();
						e.getBoard()->setValidFalse();
					}
				}
				
				while(e.getBoard()->getTurn() == 'X' )
				{
					if(choice_difficulty == "EASY")
						e.AI(17);
					else if(choice_difficulty == "MEDIUM")
						e.AI(47);
					else
						e.AI(77);
				}
				
				gameOver = e.getBoard()->isGameOver();
				e.getBoard()->setValidFalse();
				e.getBoard()->snapshot(record, *e.getBoard());
			}
		}
		
		else if(choice == 2)
		{
			while (gameOver != true) {
				gameOver = e.getBoard()->isGameOver();
				
				while(e.getBoard()->getTurn() == 'O') {
					e.getBoard()->displayBoard();//Display the board on the server
					string boardState = e.getBoard()->boardToString();
					final_move = e.getBoard()->boardToString();
					write(newsockfd, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
					e.AI(7);
				}
				
				e.getBoard()->displayBoard();
				sleep(1);
				
				while(e.getBoard()->getTurn() == 'X' ) {
					e.getBoard()->displayBoard();//Display the board on the server
					string boardState = e.getBoard()->boardToString();
					final_move = e.getBoard()->boardToString();
					write(newsockfd, boardState.c_str(), boardState.length());//Display the board to the client (line by line)
					e.AI(7);
				}
				
				gameOver = e.getBoard()->isGameOver();
			}
		}
		
		final_move = e.getBoard()->boardToString();
		string game_over = "\n\nGAME OVER!!!\n";
		write(newsockfd, final_move.c_str(), final_move.length()); //Display the board to the client (line by line)
		write(newsockfd, game_over.c_str(), game_over.length()); //Tell the client the game is over
		cout << game_over;
		usleep(1); 
		close(newsockfd);
		close(sockfd);
		break;
	 }
	 return 0; 
}
